package classwork;

import java.io.*;
import java.nio.*;
import javax.media.opengl.*;
import jocode.*;
import jomodel.*;

/**
 * Use a GLMeshRenderer object to load and render an OBJ mesh with 
 * face groups, materials and texture.  The GLMeshRenderer class can render 
 * complicated objects that include multiple materials and textures.
 * 
 * Also uses GLApp as a base class to provide init() initDisplay() run() etc.
 */
public class GLART_7_model extends JOApp {
    // holds mesh and materials, draws the mesh with groups
    JOModel model;
    float rotation = 0;
    float cameraY = 7;
    float cameraZ = 10;
    
    /**
     * Main function just creates and runs the application.
     */
    public static void main(String args[]) {
    	GLART_7_model app = new GLART_7_model();
    	displayWidth = 800;
    	displayHeight = 600;
    	//fullScreen = true;
        app.run();
    }

    /**
     * Initialize
     */
    public void setup() {
        // color of light source
        float lightDiffuse[]  = { .9f, .9f, .9f, 1f }; // direct light
        float lightSpecular[] = { .9f, .9f, .9f, 1f }; // highlight
        float lightAmbient[]  = { .4f, .4f, .45f, 1f }; // scattered light
        // light position: if last value is 0, then this describes light direction.  If 1, then light position.
        float lightPosition[] = { -4f, 4f, 6, 0f };

        // make sure OpenGL correctly layers objects
        gl.glEnable(GL.GL_DEPTH_TEST);

        // OpenGL won't draw backward facing triangles ("back faces")
        gl.glEnable(GL.GL_CULL_FACE);

        // turn lighting on (does not create a light)
        gl.glEnable(GL.GL_LIGHTING);

        // Force normals to length 1
        gl.glEnable(GL.GL_NORMALIZE);
        
        // turn on textures
    	gl.glEnable(GL.GL_TEXTURE_2D);

        // blending (for transparency)
        gl.glEnable(GL.GL_BLEND);
        gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);

        // set the background color
        gl.glClearColor(.50f, .60f, .7f, 1);

        // Draw specular highlghts on top of textures
        gl.glLightModeli(GL.GL_LIGHT_MODEL_COLOR_CONTROL, GL.GL_SEPARATE_SPECULAR_COLOR );

        // Create a light
        setLight( GL.GL_LIGHT1, lightDiffuse, lightAmbient, lightSpecular, lightPosition );

        // Set the Projection Matrix (controls perspective)
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        glu.gluPerspective(
            45.0f,        // Field Of View
            (float)getWidth() / (float)getHeight(), // aspect ratio
            1f,           // near Z clipping plane
            1000.0f);     // far Z clipping plane

        //----------------------------------------------------------------
        // Load the model (uncomment one row below)
        //----------------------------------------------------------------
        //model = new JOModel("models/JetFire/JetFire.obj");       cameraY=2;    cameraZ=3;   
        //model = new JOModel("models/escocity_patrol_cop_obj/escocity_patrol_cop_.obj");   cameraY=7;  cameraZ = 30;
        //model = new JOModel("models/porsche.obj");     cameraY=10;    cameraZ = 100;
        //model = new JOModel("models/f-16.obj");     cameraY=10;    cameraZ = 13;
        model = new JOModel("models/soccerball.obj");     cameraY=5;    cameraZ = 15;

        if (model.mesh.materialLibeName != null && model.mesh.materialLibeName.equals("porsche.mtl")) {  
        	// the porsche needs smoothing
        	model.mesh.setSmoothingAngle(60);
        	model.mesh.regenerateNormals();
        }

        // convert model to a displaylist to optimize
        model.makeDisplayList();
    }

    /**
     * Render the scene.
     */
    public void draw() {
    	rotation += .3f;

		// Clear screen and depth buffer
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

        // Select The Modelview Matrix (controls model orientation)
        gl.glMatrixMode(GL.GL_MODELVIEW);

        // reset the coordinate system to center of screen
        gl.glLoadIdentity();

        // Where is the 'eye'
        glu.gluLookAt(
        	0f, cameraY, cameraZ,   // eye position
            0f, 0f, 0f,    // target to look at
            0f, 1f, 0f);   // which way is up

        gl.glRotatef(rotation, 0,1,0);

        model.render();  
    }
}
